ZXBevents = {}
ZXBevents.hittracker = {};
ZXBevents.breakChances = {["WoodenBolt"] = ZXBconfig.woodenBoltBreakChance, ["AluminumBolt"] = ZXBconfig.aluminumBoltBreakChance}
--
-- How this works
--
-- When a zombie is hit it will register the hit on the weapon (the id of the zombie that was hit)
-- This way if multiple zombies are hit by one bolt only the last hit is registered
--
-- On weapon reload or racking, we add the zombie id of the registered hit to the hittracker together with the ammotype that was used
--
-- When a zombie dies we can't add the bolt to his inventory yet because a seconds zombie might be hit 
-- by the same bolt after the zombie dies.
-- So we copy some data to the square where the zed died because the zed object is gone
--
-- When the player approaches the gridsquare with the zombie corpse, we read the hittracker and add all
-- bolts which hit that particular zombie to the zombie corpse
--
ZXBevents.OnHitAddBolt = function(wielder, char, weapon, damage)
	local zedId = char:getID();	
	local modData = wielder:getPrimaryHandItem():getModData();
	if isServer() and modData.ctype == 'crossbow' then
		sendServerCommand('ZXB', 'registerHit', { ["zedId"] = zedId });	
	end	
	modData.lastHitZedId = zedId;
	char:getModData().hitWeapon = wielder:getPrimaryHandItem();
	if modData.ctype == 'crossbow' then
		wielder:getXp():AddXP(Perks.Aiming, ZXBconfig.xpOnHit);
		if ZXBconfig.debug then
			print(ZXBconfig.xpOnHit..' XP granted to Aiming');
		end
	end
	if ZXBconfig.debug and modData.loadedAmmo then 
		print("[OnHitAddBolt] Hit Zombie [ID: ".. zedId.."] with "..modData.loadedAmmo);
	end
end
ZXBevents.OnFire = function(wielder, char, weapon, damage)
	local modData = wielder:getPrimaryHandItem():getModData();
	if ZXBconfig.debug and modData.loadedAmmo then
		print("[OnFire] Hit Zombie [ID: ".. char:getID().."] with "..modData.loadedAmmo);
	end
	if modData.loadedAmmo ~= 'FireBolt' then return end
	if ZombRand(100) < ZXBconfig.fireBoltIgniteChance then
		char:SetOnFire();
	end
end

ZXBevents.deadZed = function(zed)
	local zedId = zed:getID();
	local square = zed:getSquare();
	square:getModData().zedId = zedId;	
	square:getModData().hitWeapon = zed:getModData().hitWeapon;
	zed:getModData().hitWeapon = nil;
	if ZXBconfig.debug then
		print("[deadZed2] Dead Zombie ID: "..zedId.." Square"..square:getID());
	end
	
end

ZXBevents.onFill = function(item, roomName, container)
    local modData = container:getSourceGrid():getModData();
	local zedId = modData.zedId;
	local weapon = modData.hitWeapon;	
	if not zedId or not weapon then return end
	if weapon:getModData().lastHitZedId then  
		ZXBevents:updateHitTracker(weapon:getModData().lastHitZedId, weapon:getModData().loadedAmmo);
		weapon:getModData().lastHitZedId = nil;	
	end
	
	if zedId and ZXBevents.hittracker[zedId] then		
		
		for k,v in pairs(ZXBevents.hittracker[zedId]) do
			local rand = ZombRand(100);
			if rand < ZXBevents.breakChances[v] then
				container:AddItem("ZXB.Broken"..v);			
			else
				container:AddItem("ZXB."..v);
			end
			if ZXBconfig.debug then
				print("[onFill] Dead Zombie [ID: "..zedId.."] Random roll: "..rand.." added item: "..v);
			end
		end
		ZXBevents.hittracker[zedId] = nil;
		modData.hitWeapon = nil;
	end
end

function ZXBevents:updateHitTracker(zedId, ammo)	
	if not zedId or not ammo then return end
	if ZXBconfig.debug then
		print("Updating Hit Tracker of Zed [ID:"..zedId.."] /Ammo [Type: "..ammo.."]");
	end
	if not ZXBevents.hittracker[zedId] then
		ZXBevents.hittracker[zedId] = {}
	end
	-- We don't keep track of Fire and Explosive Bolts, they are gone forever
	if ammo == 'WoodenBolt' or ammo == 'AluminumBolt' then
		table.insert(ZXBevents.hittracker[zedId], ammo);	
	end
end


Events.OnWeaponHitCharacter.Add(ZXBevents.OnHitAddBolt);
Events.OnWeaponHitCharacter.Add(ZXBevents.OnFire);
Events.OnZombieDead.Add(ZXBevents.deadZed);
Events.OnFillContainer.Add(ZXBevents.onFill);
